# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
from builtins import bytes  # noqa
from builtins import str  # noqa

# Third Party
from mock import MagicMock, Mock, patch

# AMP Renderer
from django_amp_renderer.middleware import AMPRenderingMiddleware

settings = Mock()
settings.AMP_RUNTIME_VERSION = '0123456789'
settings.AMP_RUNTIME_STYLES = 'body{background:pink;}'


@patch('django_amp_renderer.middleware.settings', new=settings)
class TestMiddleware:
    BOILERPLATE_HEADER_KEY = 'Boilerplate-Status'

    NON_TRIGGERING = '<!doctype html><html><head></head><body></body></html>'.encode('utf-8')
    TRIGGERING = [html.encode('utf-8') for html in [
        """
            <!doctype html>
            <html>
                <head>
                    <script async src="https://cdn.ampproject.org/v0.js" ></script>
                </head>
                <body>
                </body>
            </html>
        """,

        """
            <!doctype html>
            <html>
                <head>
                    <script async src="https://cdn.ampproject.org/v0.js" ></script>
                </head>
                <body>
                </body>
            </html>
        """,

        """
            <!doctype html>
            <html>
                <head>
                    <script async src="https://cdn.ampproject.org/v0.js" ></script>
                </head>
                <body>
                </body>
            </html>
        """,

        """
            <!doctype html>
            <html>
                <head>
                    <script type='module' async src="https://cdn.ampproject.org/lts/v0.js"></script>
                </head>
                <body>
                </body>
            </html>
        """,
    ]]

    def _get_response():
        return None

    def setup_method(self, method):
        request = Mock()
        request.META = MagicMock()
        request.GET = {}
        self.request = request

        response = MagicMock()
        response.has_header = MagicMock(return_value=True)
        headers = {'Content-Type': 'text/html'}
        response.__getitem__.side_effect = headers.__getitem__
        self.response = response

        self.middleware = AMPRenderingMiddleware(self._get_response)

    def teardown_method(self, method):
        del self.request
        del self.response
        del self.middleware

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_non_triggering(self, MockAMPRenderer):  # noqa
        self.response.content = self.NON_TRIGGERING

        self.middleware.process_response(self.request, self.response)
        assert not MockAMPRenderer.called
        self.response.__setitem__.assert_not_called()

    def _setup_renderer(self, MockAMPRenderer):  # noqa
        self.renderer = MagicMock()
        self.renderer.render = MagicMock(return_value='Yay! HTML!')
        MockAMPRenderer.return_value = self.renderer

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_triggering_1(self, MockAMPRenderer):  # noqa
        self._setup_renderer(MockAMPRenderer)

        self.response.content = self.TRIGGERING[0]

        self.middleware.process_response(self.request, self.response)

        assert MockAMPRenderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(self.BOILERPLATE_HEADER_KEY, 'Removed')

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_triggering_2(self, MockAMPRenderer):  # noqa
        self._setup_renderer(MockAMPRenderer)

        self.response.content = self.TRIGGERING[1]

        self.middleware.process_response(self.request, self.response)

        assert MockAMPRenderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(self.BOILERPLATE_HEADER_KEY, 'Removed')

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_triggering_3(self, MockAMPRenderer):  # noqa
        self._setup_renderer(MockAMPRenderer)

        self.response.content = self.TRIGGERING[2]

        self.middleware.process_response(self.request, self.response)

        assert MockAMPRenderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(self.BOILERPLATE_HEADER_KEY, 'Removed')

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_triggering_4(self, MockAMPRenderer):  # noqa
        self._setup_renderer(MockAMPRenderer)

        self.response.content = self.TRIGGERING[3]

        self.middleware.process_response(self.request, self.response)

        assert MockAMPRenderer.called
        self.renderer.render.assert_called_once()
        self.response.__setitem__.assert_called_with(self.BOILERPLATE_HEADER_KEY, 'Removed')

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_not_html(self, MockAMPRenderer):  # noqa
        self.response.content = self.TRIGGERING[0]

        headers = {'Content-Type': 'application/json'}
        self.response.__getitem__.side_effect = headers.__getitem__

        self.middleware.process_response(self.request, self.response)

        assert not MockAMPRenderer.called
        self.response.__setitem__.assert_not_called()

    @patch('django_amp_renderer.middleware.AMPRenderer')
    def test_non_unicode(self, MockAMPRenderer):  # noqa
        self.response.content = 'õ'.encode('cp857')

        self.middleware.process_response(self.request, self.response)

        assert not MockAMPRenderer.called
        self.response.__setitem__.assert_not_called()



# -*- coding: UTF-8 -*-
from __future__ import absolute_import, unicode_literals

# Standard Library
import re
from builtins import bytes  # noqa
from builtins import str  # noqa

# Third Party
from amp_renderer import AMPRenderer

# Django
from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from django.utils.encoding import DjangoUnicodeDecodeError


class AMPRenderingMiddleware(MiddlewareMixin):
    # These will be passed along to AMP Renderer
    should_strip_comments = False
    should_trim_attrs = False

    def process_response(self, request, response):
        if not response.has_header('Content-Type') or 'text/html' not in response['Content-Type']:
            return response

        try:
            content = response.content.decode('utf-8').strip()
        except (DjangoUnicodeDecodeError, UnicodeDecodeError):
            pass
        else:
            """If the script is included in the document, then apply the
            transformations that the script would eventually happen on the
            client.

            Caveats:
                This only applies the middleware if AMP v0 is included directly
                from the https://cdn.ampproject.org/v0.(m?)js. If using RTVs or
                some other method, this won’t apply as written.

                This regex doesn’t check that quotes are balanced, or fully
                confirm that the import script is valid.
            """

            attribute_regexes = [
                r"""async""",
                r"""type=['"]module['"]""",
            ]

            regex = \
                r"""<script({opt})*\s+src=['"]https://cdn\.ampproject\.org/(lts/)?v0\.m?js['"]""" \
                r"""({opt})*\s*>\s*</script>""".format(
                    opt='|'.join(r'\s+{}'.format(r) for r in attribute_regexes))

            if re.search(regex, content):
                boilerplate_header = 'Ignored'

                parser = AMPRenderer(
                    runtime_version=settings.AMP_RUNTIME_VERSION,
                    runtime_styles=settings.AMP_RUNTIME_STYLES)

                parser.should_strip_comments = self.should_strip_comments
                parser.should_trim_attrs = self.should_trim_attrs
                content = parser.render(content)

                response.content = content
                response['Content-Length'] = len(response.content)

                if parser.no_boilerplate:
                    boilerplate_header = 'Removed'

                """Set a header on the response for downstream code to know
                whether the boilerplate was removed."""
                response['Boilerplate-Status'] = boilerplate_header

        return response
